@@ -127,7 +127,7 @@ namespace xt
     inline void check_element_index(const S& shape, It first, It last)
     {
         auto dst = static_cast<typename S::size_type>(last - first);
-        It efirst = last - std::min(shape.size(), dst);
+        It efirst = last - static_cast<std::ptrdiff_t>(std::min(shape.size(), dst));
         std::size_t axis = 0;
         while (efirst != last)
         {
