@@ -94,7 +94,7 @@ namespace xt
         };
 
         template <class F, class R>
-        struct simd_return_type<F, R, void_t<decltype(&F::simd_apply)>>
+        struct simd_return_type<F, R, void_t<decltype(&F::template simd_apply<R>)>>
         {
             using type = R;
         };
@@ -109,6 +109,13 @@ namespace xt
             using simd_type = xsimd::simd_type<T>;
         };
 
+        template <class T>
+        struct functor_return_type<T, std::complex<T>>
+        {
+            using type = std::complex<T>;
+            using simd_type = xsimd::simd_type<std::complex<T>>;
+        };
+
         template <class T>
         struct functor_return_type<T, bool>
         {
@@ -145,7 +152,7 @@ namespace xt
      * xfunction_base *
      ******************/
 
-#define DL DEFAULT_LAYOUT
+#define DL XTENSOR_DEFAULT_LAYOUT
 
     /**
      * @class xfunction_base
@@ -286,7 +293,7 @@ namespace xt
 
     protected:
 
-        template <class Func, class... CTA, class U = std::enable_if_t<!std::is_base_of<Func, self_type>::value>>
+        template <class Func, class... CTA, class U = std::enable_if_t<!std::is_base_of<std::decay_t<Func>, self_type>::value>>
         xfunction_base(Func&& f, CTA&&... e) noexcept;
 
         ~xfunction_base() = default;
@@ -467,8 +474,6 @@ namespace xt
 
         reference operator*() const;
 
-        bool equal(const self_type& rhs) const;
-
     private:
 
         template <std::size_t... I>
@@ -478,14 +483,6 @@ namespace xt
         std::tuple<typename std::decay_t<CT>::const_stepper...> m_it;
     };
 
-    template <class F, class R, class... CT>
-    bool operator==(const xfunction_stepper<F, R, CT...>& it1,
-                    const xfunction_stepper<F, R, CT...>& it2);
-
-    template <class F, class R, class... CT>
-    bool operator!=(const xfunction_stepper<F, R, CT...>& it1,
-                    const xfunction_stepper<F, R, CT...>& it2);
-
     /*************
      * xfunction *
      *************/
@@ -511,7 +508,7 @@ namespace xt
         using self_type = xfunction<F, R, CT...>;
         using base_type = xfunction_base<F, R, CT...>;
 
-        template <class Func, class... CTA, class U = std::enable_if_t<!std::is_base_of<Func, self_type>::value>>
+        template <class Func, class... CTA, class U = std::enable_if_t<!std::is_base_of<std::decay_t<Func>, self_type>::value>>
         xfunction(Func&& f, CTA&&... e) noexcept;
 
         ~xfunction() = default;
@@ -1074,12 +1071,6 @@ namespace xt
         return deref_impl(std::make_index_sequence<sizeof...(CT)>());
     }
 
-    template <class F, class R, class... CT>
-    inline bool xfunction_stepper<F, R, CT...>::equal(const self_type& rhs) const
-    {
-        return p_f == rhs.p_f && m_it == rhs.m_it;
-    }
-
     template <class F, class R, class... CT>
     template <std::size_t... I>
     inline auto xfunction_stepper<F, R, CT...>::deref_impl(std::index_sequence<I...>) const -> reference
@@ -1087,20 +1078,6 @@ namespace xt
         return (p_f->m_f)(*std::get<I>(m_it)...);
     }
 
-    template <class F, class R, class... CT>
-    inline bool operator==(const xfunction_stepper<F, R, CT...>& it1,
-                           const xfunction_stepper<F, R, CT...>& it2)
-    {
-        return it1.equal(it2);
-    }
-
-    template <class F, class R, class... CT>
-    inline bool operator!=(const xfunction_stepper<F, R, CT...>& it1,
-                           const xfunction_stepper<F, R, CT...>& it2)
-    {
-        return !(it1.equal(it2));
-    }
-
     /****************************
      * xfunction implementation *
      ****************************/
